
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _MOOF_CAMERA_HH_
#define _MOOF_CAMERA_HH_

#include <Moof/Event.hh>
#include <Moof/Frustum.hh>
#include <Moof/Math.hh>
#include <Moof/RigidBody.hh>


namespace Mf {


class Camera : public RigidBody3
{
public:

	Camera()
	{
		mState.init();
		mPrevState.init();

		cml::quaternion_rotation_world_y(mState.orientation, SCALAR(0.0));
	}

	void setPosition(const Vector3& position);
	void setRotation(const Quaternion& rotation);

	void lookAt(const Vector3& point);

	void setProjection(const Matrix4& projection);
	void setProjection(Scalar fovy, Scalar aspect, Scalar near, Scalar far);

	const Matrix4& getModelview() const
	{
		return mModelview;
	}

	const Matrix4& getProjection() const
	{
		return mProjection;
	}

	const Frustum& getFrustum() const
	{
		return mFrustum;
	}


	void uploadToGL(Scalar alpha = 0) const;

	void update(Scalar t, Scalar dt);
	void draw(Scalar alpha = 0) const;
	void handleEvent(const Event& event);

private:

	void calculate(Scalar alpha) const;

	mutable Matrix4 	mModelview;
	Matrix4				mProjection;

	mutable Frustum		mFrustum;
};


} // namespace Mf

#endif // _MOOF_CAMERA_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

